/*-------------------------------------------------------------------------------------------------\
| Misc CAN Node Firmware
|
| File:					MSC.C			Mainline Code
|
| Global Routines:	Main()
|
| Local Routines:		
|
| Discussion:			This firmware runs on a PIC16F877 used in the RiteAngle III Logic Module.
|							For more detailed information, please see the "RA3" documentation.
|
|   Date     | Notes
|------------|---------------------------------------------------------------------------------------
| 01/05/2002 | Created.!
\-------------------------------------------------------------------------------------------------*/

#include	<pic.h>
#include "msc.h"

//-------------------------------------------------------------------------------------------------
//  Define Global Variables (FOR ALL ROUTINES AND MODULES)
//-------------------------------------------------------------------------------------------------

	// Note: 1) In PICC, "char" is an byte which is by default unsigned, "unsigned int" is a 16bit quantity.
	// 		2) ALL VARIABLES ARE INITIALIZED TO ZERO ON RESET (courtesy Hi-Tech PIC C startup code)

	bit				gbPowerUp;								//* 1 = Power up flagged, do LED flashing / 0 = Power up mode off
	char				gcSystemError;							// 0 = System ok, 1 = General System ("internal") error

	bit				gbOneMs;									// 1 = it has been 1ms / 0 = not yet
	bit				gbTenMs;									// 1 = it has been 10ms / 0 = not yet
	bit				gbHundredMs;							// 1 = it has been 100ms / 0 = not yet
	bit				gbOneSec;								// 1 = it has been 1 sec / 0 = not yet

	bit				gbSJAInterrupt;						// 1 = The SJA has interrupted the PIC, 0 = it hasn't
	bit				gbTxReady;								// 1 = The SJA is ready to transmit a message, 0 = busy
	bit				gbRxReady;								// 1 = The SJA has received a message, 0 = Nothing waiting

	char				gcTxQueueOut; 							// Index for the next message to be read out of the transmit queue
	char				gcTxQueueIn;							// Index for the next message to load into the transmit queue
	bank2 struct message	gmTransmitQueue[TXQUEUELENGTH];	// Queue for outgoing CAN messages 

	bank3 char		cSJARegisters[32];					// Global array to "peek" into SJA1000

	struct message	gmMessage;								// Latest message from the CAN bus

	char				gcErrArray[8];
	char				gcErrIndex;

//-------------------------------------------------------------------------------------------------	
//  Define Local (to this file) Global Variables
//-------------------------------------------------------------------------------------------------

	// None
	
//-------------------------------------------------------------------------------------------------	
//  Local #defines
//-------------------------------------------------------------------------------------------------

	// None

//-------------------------------------------------------------------------------------------------	
//  Local Function Prototypes
//-------------------------------------------------------------------------------------------------

	void	ProcessMessage (void);

/*-------------------------------------------------------------------------------------------------\
| Function:		main
|
| Purpose:		Run all Mainline code
|
| Discussion:	Main calls the initialization code and then goes into an infinite loop of calling
|					all the appropriate main line functions:
|
| Expects:		Interrupts to be enabled.
|					gbOneMs,gbHundredMs to be set by Timer2()
|					
|   Date     | Notes
|------------|---------------------------------------------------------------------------------------
| 01/05/2002 | Stolen from ADG code.
\-------------------------------------------------------------------------------------------------*/

void main (void)
{
	// Initialize the PIC and the SJA1000
	
	Initialize();
	Init_SJA();

	// ------------------------------------------------------------------------
	
	while (TRUE)
	{
		// If the SJA has interrupted the PIC then find out why
		
		if (gbSJAInterrupt)
			ServiceSJA();

		// If the SJA is ready to send a message and there are messages to send,
		// send them.

		if ( gcTxQueueIn != gcTxQueueOut )
			SendMessage();

		// If the SJA has messages waiting to be read, then get them and process them.
			
		if ( gbRxReady )
		{
			GetMessage();
			ProcessMessage();
		}

		// Call all functions which run at 1KHz (every 1ms)

		if (gbOneMs)
		{
			gbOneMs = 0;
		}

		// Call the mainline functions at 100Hz (every 10ms)

		if (gbTenMs)
		{
			gbTenMs = 0;
		}
		
		// Call the mainline functions at 10Hz (every 100ms)
		
		if (gbHundredMs)
		{
			ButtonPushed();
			gbHundredMs = 0;
		}

		// Call the mainline functions at 1Hz (every second)

		if (gbOneSec)
		{
			SendCount();
			gbOneSec = 0;
		}
	}
}




/*-------------------------------------------------------------------------------------------------\
| Function:		ProcessMessage
|
| Purpose:		Process the latest CAN message read in from the SJA1000.
|
| Discussion:	Do stuff
|             
| Expects:		
|
|   Date     | Notes
|------------|---------------------------------------------------------------------------------------
| 01/07/2002 | Created.
\--------------------------------------------------------------------------------------------------*/
		
void	ProcessMessage (void)
{
	// ------------------------------------------------------------------------
	// Message ID 0x7FE = 0b111 1111 1110
	
	if ( (gmMessage.cMsgID1 == 0b11111111) && ((gmMessage.cMsgID2 & 0b11100000) == 0b11000000) )
	{
		if (gmMessage.cData[0])
		{
			#ifdef STATUSLEDS
				GREEN1_LED = LED_ON;
			#endif
		}
		else
		{
			#ifdef STATUSLEDS
				GREEN1_LED = LED_OFF;
			#endif
		}		
	}

	// ------------------------------------------------------------------------
	// Message ID 0x7FF = 0b111 1111 1111

	else if ( (gmMessage.cMsgID1 == 0b11111111) && ((gmMessage.cMsgID2 & 0b11100000) == 0b11100000) )
	{
		if (gmMessage.cData[0])
		{
			#ifdef STATUSLEDS
				GREEN2_LED = LED_ON;
			#endif
		}
		else
		{
			#ifdef STATUSLEDS
				GREEN2_LED = LED_OFF;
			#endif
		}
	}

	// ------------------------------------------------------------------------
	// We received a system message we don't know what to do with, so ignore it.

	else
	{
		#ifdef STATUSLEDS
			OVERFLOW_LED = LED_ON;
			CANERR_LED = LED_ON;
			BUSERR_LED = LED_ON;
		#endif
	}
}





