/*-------------------------------------------------------------------------------------------------\
| Misc CAN Node Firmware	
|
| File:					MSC_INT.C		Interrupt Routines
|
| Global Routines:	ISR()
|
| Local Routines:		Timer2()
|
| IMPORTANT NOTE ON THE STRUCTURE OF THIS MODULE:
| -----------------------------------------------
| PICC will automatically save the context of all functions that an "interrupt"-labeled routine
| (ISR_LM below) calls. However, the functions called must be ABOVE THE INTERRUPT-LABELED FUNCTION
| in the same code module (i.e., the same file). Thus, this module must RETAIN THE ORDER OF THE
| FUNCTIONS with the interrupt function being last in the module. For more information, see the
| Hi-Tech PIC C Manual.
\-------------------------------------------------------------------------------------------------*/

#include	<pic.h>
#include "msc.h"


//-------------------------------------------------------------------------------------------------
//  Define Locally Global Variables (variables accessible by functions in this file only)
//-------------------------------------------------------------------------------------------------

	// None 

//-------------------------------------------------------------------------------------------------
//  Local #defines
//-------------------------------------------------------------------------------------------------

	// None

//-------------------------------------------------------------------------------------------------	
//  Local Function Prototypes
//-------------------------------------------------------------------------------------------------

	void	Timer2	(void);
	void	RB0Int	(void);


/*-------------------------------------------------------------------------------------------------\
| Function:  	Timer2
|
| Purpose:   	1kHz interrupt
|
| Discussion:	Provide a time base to the uC
|
| Expects:		To be called on the 1KHz Timer2 interrupt
|					Interrupts to be disabled.
|					
|   Date     | Notes
|------------|---------------------------------------------------------------------------------------
| 01/05/2002 | Stolen from ADG code.
\--------------------------------------------------------------------------------------------------*/

void Timer2 (void)
{
	static	char	cOneMsClock;
	static	char	cTenMsClock;
	static	char	c100MsClock;
	
	// Clear the Timer2 interrupt
	
	TMR2IF = 0;

	// Set the global flag that tells Main it has been 1ms

	gbOneMs = 1;
	
	// Increment the local clock and set the global 100ms flag to tell main it has been 100ms
		
	++cOneMsClock;
	if (cOneMsClock >= 99)
	{
		// Reset the local count in milliseconds and flag the main routine that it has been 100ms.
		// Flag an error if Main hasn't cleared the bit yet.
		
		cOneMsClock = 0;

		if (gbHundredMs)
		{
			gcSystemError = 6;

			#ifdef	STATUSLEDS
				ERROR_LED = LED_ON;
			#endif
		}
		else
			gbHundredMs = 1;

		// Count to one second
		++c100MsClock;
		if (c100MsClock >= 9)
		{
			c100MsClock = 0;
			gbOneSec = 1;
		}
	}

	++cTenMsClock;
	if (cTenMsClock >= 9)
	{
		cTenMsClock = 0;
		gbTenMs = 1;
	}
}



/*-------------------------------------------------------------------------------------------------\
| Function:  	RB0Int
|
| Purpose:   	Check to see if a peripheral chip is interrupting us
|
| Discussion:	It's the CAN chip. Check for received messages, errors, etc. It would be nice to
|					put this in the SJA file, but it must be here in case there are weird context save
|					issues (routine must be above "interrupt" function).
|
| Expects:		Interrupts to be disabled.
|					
|   Date     | Notes
|------------|---------------------------------------------------------------------------------------
| 01/05/2002 | Stolen from ADG code.
\--------------------------------------------------------------------------------------------------*/

void	RB0Int	(void)
{
	// Clear the interrupt
	
	INTF = 0;

	// Set a global flag telling the mainloop to grab data

	gbSJAInterrupt = 1;
}

/*-------------------------------------------------------------------------------------------------\
| Function:  	ISR
|
| Purpose:   	ISR Is the interrupt service routine.
|
| Discussion:	
|
| Expects:		To be called on any kind of PIC16F877 interrupt.
|					For global interupts to be disabled by the interrupt (INTSTA, GIE = 0).
|					Hi-Tech C to save and restore all variables used in and by these int. routines.
|					
|   Date     | Notes
|------------|---------------------------------------------------------------------------------------
| 01/05/2002 | Stolen from ADG code.
\--------------------------------------------------------------------------------------------------*/


void interrupt ISR (void)
{
	// Now we determine who interrupted us and go to that ISR_LM. Note that the order of checking
	// the interrupt flags determines the interrupt priority.

	if (!TMR2IF && !INTF)
	{
		// The cause of the interrupt is unknown; that's important enough to flag as a system error.
		
		gcSystemError = 1;
		
		#ifdef STATUSLEDS
			ERROR_LED = LED_ON;
		#endif
	}
	else
	{
		if (TMR2IF)		// Check the Timer2 interrupt
			 Timer2();

		if (INTF)		// Check for a RB0/INT intterupt
			RB0Int();			
	}
	
	// Note: This routine Automatically RETFIEs (which enables interrupts)
}

